﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;

namespace Trolleybus
{
    internal class DrawningObjectTrolleybus : IDrawningObject
    {
        public DrawingTrolleybus _trolleybus { get; private set; }

        public DrawningObjectTrolleybus(DrawingTrolleybus trolleybus)
        {
            _trolleybus = trolleybus;
        }

        public float Step => _trolleybus?.Trolleybus?.Step ?? 0;

        public DrawingTrolleybus GetTrolleybus => _trolleybus;

        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return _trolleybus?.GetCurrentPosition() ?? default;
        }

        public void MoveObject(Direction direction)
        {
            _trolleybus?.MoveTransport(direction);
        }

        public void nothing()
        {
            throw new NotImplementedException();
        }

        public void SetObject(int x, int y, int width, int height)
        {
            _trolleybus.SetPosition(x, y, width, height);
        }

        void IDrawningObject.DrawningObject(Graphics g)
        {
            _trolleybus.DrawTransport(g);
        }

        public string GetInfo() => _trolleybus?.GetDataForSave() ?? string.Empty;

        public static IDrawningObject Create(string data) => new DrawningObjectTrolleybus(data.CreateDrawingTrolleybus());
        public bool Equals(IDrawningObject? other)
        {
            if (other is not DrawningObjectTrolleybus otherAirplane)
            {
                return false;
            }
            var entity = _trolleybus.Trolleybus;
            var otherEntity = otherAirplane._trolleybus.Trolleybus;
            if (entity.GetType() != otherEntity.GetType() ||
                entity.Speed != otherEntity.Speed ||
                entity.Weight != otherEntity.Weight ||
                entity.BodyColor != otherEntity.BodyColor)
            {
                return false;
            }
            if (entity is EntitySmallTrolleybus entitySmallTrolleybus &&
                otherEntity is EntitySmallTrolleybus otherEntityAirplaneWithRadar && (
                entitySmallTrolleybus.BodyKit != otherEntityAirplaneWithRadar.BodyKit ||
                entitySmallTrolleybus.DopColor != otherEntityAirplaneWithRadar.DopColor ||
                entitySmallTrolleybus.Horns != otherEntityAirplaneWithRadar.Horns ||
                entitySmallTrolleybus.Battary != otherEntityAirplaneWithRadar.Battary))
            {
                return false;
            }
            return true;
        }
    }
}
