﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Trolleybus
{
    internal class SetTrolleybusGeneric<T>
        where T : class, IEquatable<T>
    {
        private readonly List<T> _places;
        public int Count => _places.Count;

        private readonly int _maxCount;
        public SetTrolleybusGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T>();
        }
        private bool CorrectPos(int pos)
        {
            return 0 <= pos && pos < _maxCount;
        }
        public int Insert(T trolleybus)
        {
            if (_places.Count > _maxCount)
            {
                return -1;
            }
            return Insert(trolleybus, 0);
        }
        public int Insert(T trolleybus, int position)
        {
            if (!CorrectPos(position))
            {
                return -1;
            }

            _places.Insert(position, trolleybus);
            return position;
        }
        public T Remove(int position)
        {
            if (!CorrectPos(position))
                return null;
            T temp = _places[position];
            _places.RemoveAt(position);
            return temp;
        }
        public T this[int position]
        {
            get
            {
                return CorrectPos(position) && position < Count ? _places[position] : null;
            }
            set
            {
                Insert(value, position);
            }
        }
        public IEnumerable<T> GetTrolleybus()
        {
            foreach (var trolleybus in _places)
            {
                if (trolleybus != null)
                {
                    yield return trolleybus;
                }
                else
                {
                    yield break;
                }
            }
        }

        public void SortSet(IComparer<T> comparer)
        {
            if (comparer == null)
            {
                return;
            }
            _places.Sort(comparer);
        }
    }
}